﻿using System.Collections;
using UnityEngine;

[RequireComponent(typeof(SpriteRenderer))]
[RequireComponent(typeof(BoxCollider2D))]
public class CoreBehaviour : MonoBehaviour
{
    public int Resolution = 50;

    private GameMaster gameMaster;
    private SpriteRenderer spriteRenderer;
    private bool changeColor;

    void Start()
    {
        changeColor = true;
        gameMaster = GameObject.Find("_GameMaster").GetComponent<GameMaster>();
        spriteRenderer = GetComponent<SpriteRenderer>();

        StartCoroutine(SetCoreColor());
    }

    IEnumerator SetCoreColor()
    {
        while (changeColor)
        {
            // set the original texture, core has to be all white again in order to re-draw black pixels
            spriteRenderer.sprite = Resources.Load<Sprite>("Sprites/Core");

            // generate new texture, with X % of black pixels
            Texture2D newTexture = new Texture2D(Resolution, Resolution);
            for (int y = 0; y < newTexture.height; y++)
            {
                for (int x = 0; x < newTexture.width; x++)
                {
                    if (Random.Range(0, 101) < gameMaster.MaxHealth - gameMaster.Health)
                        newTexture.SetPixel(x, y, Color.black);
                    else
                        newTexture.SetPixel(x, y, Color.white);
                }
            }

            newTexture.Apply();
            // pivot (0.5, 0.5) means that it's in the center
            spriteRenderer.sprite = Sprite.Create(newTexture, new Rect(0, 0, newTexture.width, newTexture.height),
                new Vector2(0.5f, 0.5f));

            yield return new WaitForSeconds(4);
        }
    }

    void OnCollisionEnter2D(Collision2D collision)
    {
        if (collision.gameObject.tag == "Enemy")
        {
            gameMaster.Health -= collision.gameObject.GetComponent<EnemyBehaviour>().Enemy.Damage;
            Destroy(collision.gameObject);
            Debug.Log("Enemy was destoryed!");
        }
    }
}
