﻿using System;
using System.ComponentModel;
using UnityEngine;

[RequireComponent(typeof(BoxCollider2D))]
[RequireComponent(typeof(Rigidbody2D))]
public class EnemyBehaviour : MonoBehaviour
{
    public GameObject HitAnimation;
    public Enemy Enemy { get; private set; }
    public event EventHandler EnemyHitted;

    private GameMaster gameMaster;
    private GameInfo gameInfo;

    // Use this for initialization
	void Start ()
	{
	    gameMaster = GameObject.Find("_GameMaster").GetComponent<GameMaster>();
	    gameInfo = GameObject.Find("_GameInfo").GetComponent<GameInfo>();

        // gravity won't affect our enemies
	    GetComponent<Rigidbody2D>().gravityScale = 0;

        EnemyHitted += EnemyBehaviour_EnemyHitted;
	}

    void EnemyBehaviour_EnemyHitted(object sender, EventArgs e)
    {
        // spawn hit animation
        if (!Enemy.IsAlive)
        {
            Instantiate(HitAnimation, transform.position, Quaternion.identity);
            gameMaster.Score += 10 * gameInfo.EnemiesDifficulty;
            gameMaster.Coins += 5 * gameInfo.EnemiesDifficulty;
            Destroy(gameObject);
        }
    }

    public void SetNewEnemy(int waveNumber, float waveSpeed)
    {
        if (gameInfo == null)
            gameInfo = GameObject.Find("_GameInfo").GetComponent<GameInfo>();

        int health = (gameInfo.EnemiesDifficulty >= 3) ? (20 + ((waveNumber - 1 + gameInfo.EnemiesDifficulty) * 6)) :
            (20 + ((waveNumber - 1 + gameInfo.EnemiesDifficulty) * 2));
        int damage = waveNumber + gameInfo.EnemiesDifficulty;

        Enemy = new Enemy(health, damage, 0, 0, 0, 0, waveSpeed, eEnemy.EnemyType.Beast);
    }

    public void OnEnemyHitted()
    {
        if (EnemyHitted != null)
            EnemyHitted(this, new EventArgs());
    }
}
