﻿using System.Linq;
using UnityEngine;
using System.Collections;

[RequireComponent(typeof(SpriteRenderer))]
public class HealthBar : MonoBehaviour
{
    public int TextureWidth, TextureHeight;

    private EnemyBehaviour enemyBehaviour;
    private SpriteRenderer spriteRenderer;

    private Texture2D texture;

	// Use this for initialization
	void Start ()
	{
	    enemyBehaviour = GetComponentInParent<EnemyBehaviour>();
	    spriteRenderer = GetComponent<SpriteRenderer>();
        texture = new Texture2D(TextureWidth, TextureHeight);

        // this fires when Health was changed
        enemyBehaviour.EnemyHitted += EnemyBehaviour_EnemyHitted;
	}

    private void EnemyBehaviour_EnemyHitted(object sender, System.EventArgs e)
    {
        // enemies health is less than max so show health bar, else hide it
        if (enemyBehaviour.Enemy.Health < enemyBehaviour.Enemy.MaxHealth
            &&
            enemyBehaviour.Enemy.Health >= 0)
        {
            MakeTexture();
            spriteRenderer.sprite = Sprite.Create(texture, new Rect(0, 0, TextureWidth, TextureHeight),
                new Vector2(0.5f, 0.5f));

            spriteRenderer.enabled = true;
        }
        else
            spriteRenderer.enabled = false;
    }

    private void MakeTexture()
    {
        float remainingHealthPercent = (float)enemyBehaviour.Enemy.Health / enemyBehaviour.Enemy.MaxHealth;
        // set green pixels (remaining health)
        float greenChunkWidth = TextureWidth * remainingHealthPercent;

        // set green pixels only if chunk is bigger than 0 pixels (enemy health is more than 0)
        // otherwise greenpixels amount is 0 and 0 length array is not even possible
        if (greenChunkWidth > 0)
        {
            float amountOfGreenPixels = greenChunkWidth * TextureHeight;
            Color[] greenPixels = new Color[(int)amountOfGreenPixels];
            for (int i = 0; i < greenPixels.Length; i++)
            {
                greenPixels[i] = Color.green;
            }

            texture.SetPixels(0, 0, (int)greenChunkWidth, TextureHeight, greenPixels);
        }

        // here we need to compute redChunk to whole number and add one if we cut greenChunk short
        int redChunkWidth = TextureWidth - (int)greenChunkWidth;
        // create new array of pixels and set them all to green
        Color[] redPixels = new Color[redChunkWidth * TextureHeight];
        for (int i = 0; i < redPixels.Length; i++)
        {
            redPixels[i] = Color.red;
        }

        texture.SetPixels((int)greenChunkWidth, 0, redChunkWidth, TextureHeight, redPixels);

        texture.Apply();
    }
}
