﻿using System;
using System.Collections.Generic;
using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using Random = UnityEngine.Random;

public class SpawnerBehaviour : MonoBehaviour
{
    public Text wavesRemainingText;
    public Text nextWaveTimerText;
    public List<GameObject> SpawnableGameObjects;
    public float TimeBetweenWaves = 25;
    public float TimeBetweenSpawns = 3.5f;

    private int wavesRemaining;
    public int WavesRemaining
    {
        get { return wavesRemaining; }
        set
        {
            wavesRemaining = value;
            wavesRemainingText.text = Convert.ToString(wavesRemaining);
        }
    }

    public List<Wave> Waves { get; set; }
    public Wave CurrentWave { get; set; }
    public event EventHandler WavesGenerated, WaveEnded;

    private int wavesCount, minWaveSize, maxWaveSize;
    private int enemiesSpawned;
    private Transform tra;
    private GameInfo gameInfo;
    private int elapsedSeconds;
    private bool spawningEnemies;

	// Use this for initialization
	void Start ()
	{
        // load game settings
	    gameInfo = GameObject.Find("_GameInfo").GetComponent<GameInfo>();
	    wavesCount = gameInfo.WavesCount;
	    minWaveSize = gameInfo.WaveSizeMin;
	    maxWaveSize = gameInfo.WaveSizeMax;

        GenerateWaves();
        CurrentWave = Waves[0];
        enemiesSpawned = 0;
        WavesRemaining = wavesCount;
        elapsedSeconds = 0;
	    spawningEnemies = false;

        tra = transform;
        nextWaveTimerText.text = Convert.ToString(TimeBetweenWaves);

        // spawner position is set after generation generates the start tile

        // start waves
        Debug.Log("Starting wave coroutine.");
	    StartCoroutine(StartWave());

        // start timer
	    StartCoroutine(Timer());
	}

    protected virtual void OnWaveEnded(EventArgs e)
    {
        if (WaveEnded != null)
            WaveEnded(this, e);
    }

    protected virtual void OnWavesGenerated(EventArgs e)
    {
        if (WavesGenerated != null)
            WavesGenerated(this, e);
    }

    private void GenerateWaves()
    {
        Waves = new List<Wave>();
        for (int i = 0; i < wavesCount; i++)
        {
            Waves.Add(new Wave(
                Random.Range(0, SpawnableGameObjects.Count),
                i + 1,
                Random.Range(minWaveSize, maxWaveSize),
                Random.Range(5f, 8f)));
        }

        OnWavesGenerated(EventArgs.Empty);
        //EventManager.TriggerEvent("WavesGenerated");
    }

    IEnumerator StartWave()
    {
        yield return new WaitForSeconds(TimeBetweenWaves);

        while (CurrentWave.Number <= wavesCount)
        {
            // all enemies were spawned and current wave size is less than max waves count 
            if (enemiesSpawned == CurrentWave.Size && CurrentWave.Number < wavesCount)
            {
                OnWaveEnded(EventArgs.Empty);
                //EventManager.TriggerEvent("WaveEnded");

                spawningEnemies = false;
                yield return new WaitForSeconds(TimeBetweenWaves);
                
                enemiesSpawned = 0;
                CurrentWave = Waves[Waves.IndexOf(CurrentWave) + 1];
                WavesRemaining = wavesCount - (CurrentWave.Number - 1);
                Debug.Log("Setting up new wave: " + CurrentWave);
            }
            else
                yield return StartCoroutine(SpawnEnemy());
        }
    }

    IEnumerator SpawnEnemy()
    {
        while (enemiesSpawned < CurrentWave.Size)
        {
            if (!spawningEnemies)
                spawningEnemies = true;

            Debug.Log("Spawning new enemy.");
            // spawns a new enemy
            GameObject newEnemy = Instantiate(SpawnableGameObjects[CurrentWave.EnemyType],
                new Vector3(tra.position.x, tra.position.y, 0),
                Quaternion.identity);

            // set new enemy (enemies get stronger with each wave)
            EnemyBehaviour newEnemyBehaviour = newEnemy.GetComponent<EnemyBehaviour>();
            newEnemyBehaviour.SetNewEnemy(CurrentWave.Number, CurrentWave.Speed);

            enemiesSpawned++;

            yield return new WaitForSeconds(TimeBetweenSpawns);
        }
    }

    IEnumerator Timer()
    {
        // timer counts only when spawner ain't spawning, cause while spawning we don't
        // count time between waves yet
        while (elapsedSeconds < TimeBetweenWaves)
        {
            if (!spawningEnemies)
            {
                yield return new WaitForSeconds(1);

                elapsedSeconds++;
                nextWaveTimerText.text = Convert.ToString(TimeBetweenWaves - elapsedSeconds);

                if (elapsedSeconds == TimeBetweenWaves)
                    elapsedSeconds = 0;
            }
            else
                yield return null;
        }
    }

    public bool AllWavesFinished()
    {
        return (wavesCount - CurrentWave.Number == 0 && CurrentWave.Size - enemiesSpawned == 0);
    }
}
