﻿using UnityEngine;
using UnityEngine.UI;

[RequireComponent(typeof(RectTransform))]
public class BuildGUI : MonoBehaviour
{
    public GameObject SquareTower, CircleTower, TriangleTower, Mine;
    public Text BeamCost, HunterCost, AlchemyCost, MineCost;

	private RectTransform trans;
    private GameMaster gameMaster;
    private TileMapGraphics tgMap;
    private Tile clickedTile;

	// Use this for initialization
	void Start ()
	{
		trans      = GetComponent<RectTransform>();
        gameMaster = GameObject.Find("_GameMaster").GetComponent<GameMaster>();
        tgMap      = GameObject.Find("TileMap").GetComponent<TileMapGraphics>();
		gameObject.SetActive(false);
	}

	public void Close()
	{
		gameObject.SetActive(false);
	}

	public void Show(Tile tile)
	{
		if (tile.CanBuildOn && !tile.IsOccupied)
		{
			Vector3 newPos = Camera.main.ScreenToWorldPoint(
				new Vector3(Input.mousePosition.x + 10, Input.mousePosition.y - 15));

            trans.SetPositionAndRotation(new Vector3(newPos.x, newPos.y, -5), Quaternion.identity);
			clickedTile = tile;
            SetBuildCosts();
			gameObject.SetActive(true);
		}
		else
			gameObject.SetActive(false);
	}

    private void SetBuildCosts()
    {
        BeamCost.text = string.Format("Cost: {0}", BeamTowerUpgrades.BuildCost);
        HunterCost.text = string.Format("Cost: {0}", HunterTowerUpgrades.BuildCost);
        AlchemyCost.text = string.Format("Cost: {0}", AlchemyTowerUpgrades.BuildCost);
        MineCost.text = string.Format("Cost: {0}", GoldMineUpgrades.BuildCost);
    }

	public void BuildBeamTower()
	{
		// zde se nelze ptát na tower, protoze to neni instance
		// takze cena veze bude muset byt umistena v GameMaster scriptu
		if (gameMaster.Coins >= BeamTowerUpgrades.BuildCost)
		{
			// real position of Y on screen is negative so we gotta negate it
			Instantiate(CircleTower,
				new Vector3(clickedTile.Position.x * tgMap.tileSize, -clickedTile.Position.y * tgMap.tileSize, 0),
				Quaternion.identity);

			gameMaster.Coins -= BeamTowerUpgrades.BuildCost;
			clickedTile.IsOccupied = true;
			gameObject.SetActive(false);
			Debug.Log("Tower was built.");
		}
		else
			Debug.Log("Not enough coins.");
	}

	public void BuildHunterTower()
	{
		// zde se nelze ptát na tower, protoze to neni instance
		// takze cena veze bude muset byt umistena v GameMaster scriptu
		if (gameMaster.Coins >= HunterTowerUpgrades.BuildCost)
		{
			// real position of Y on screen is negative so we gotta negate it
			Instantiate(SquareTower,
				new Vector3(clickedTile.Position.x * tgMap.tileSize, -clickedTile.Position.y * tgMap.tileSize, 0),
				Quaternion.identity);

			gameMaster.Coins -= HunterTowerUpgrades.BuildCost;
			clickedTile.IsOccupied = true;
			gameObject.SetActive(false);
			Debug.Log("Tower was built.");
		}
		else
			Debug.Log("Not enough coins.");
	}

	public void BuildAlchemyTower()
	{
		// zde se nelze ptát na tower, protoze to neni instance
		// takze cena veze bude muset byt umistena v GameMaster scriptu
		if (gameMaster.Coins >= AlchemyTowerUpgrades.BuildCost)
		{
			// real position of Y on screen is negative so we gotta negate it
			Instantiate(TriangleTower,
				new Vector3(clickedTile.Position.x * tgMap.tileSize, -clickedTile.Position.y * tgMap.tileSize, 0),
				Quaternion.identity);

			gameMaster.Coins -= AlchemyTowerUpgrades.BuildCost;
			clickedTile.IsOccupied = true;
			gameObject.SetActive(false);
			Debug.Log("Tower was built.");
		}
		else
			Debug.Log("Not enough coins.");
	}

	public void BuildGoldMine()
	{
		// zde se nelze ptát na tower, protoze to neni instance
		// takze cena veze bude muset byt umistena v GameMaster scriptu
		if (gameMaster.Coins >= GoldMineUpgrades.BuildCost)
		{
			// real position of Y on screen is negative so we gotta negate it
			Instantiate(Mine,
				new Vector3(clickedTile.Position.x * tgMap.tileSize, -clickedTile.Position.y * tgMap.tileSize, 0),
				Quaternion.identity);

			gameMaster.Coins -= GoldMineUpgrades.BuildCost;
			clickedTile.IsOccupied = true;
			gameObject.SetActive(false);
			Debug.Log("Mine was built.");
		}
		else
			Debug.Log("Not enough coins.");
	}
}
