﻿using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using System;
using UnityEngine.UI;

public class TowerGUI : MonoBehaviour
{
    public Text TowerType, Damage, Range, FireRate, TowerLevel, UpgradeCost, UpgradeType;
    public GameObject TbxWarning, HunterStatsHolder,
		BeamStatsHolder, AlchemyStatsHolder, MineStatsHolder, UpgradeGUI;

    private GameMaster gameMaster;
    private TileMapGraphics tgMap;
    private TowerBehaviour hoveredTower, prevHoveredTower, clickedTower;
    private TowerRangeHighlighter towerHighlighter;

	// Use this for initialization
	void Start()
	{
	    prevHoveredTower = null;
        tgMap            = GameObject.Find("TileMap").GetComponent<TileMapGraphics>();
        gameMaster       = GameObject.Find("_GameMaster").GetComponent<GameMaster>();
        towerHighlighter = GameObject.Find("TowerRangeHighlighter").GetComponent<TowerRangeHighlighter>();
		// tower gui is inactive from start
		gameObject.SetActive(false);
		// warning is inactive from start
		TbxWarning.SetActive(false);
		// also specific stats
		BeamStatsHolder.SetActive(false);
		HunterStatsHolder.SetActive(false);
		AlchemyStatsHolder.SetActive(false);
		MineStatsHolder.SetActive(false);
		UpgradeGUI.SetActive(false);

        UpgradeGUI.SetActive(false);
    }

    private void SetValues()
    {
        // Set all GUI values
        TowerType.text   = clickedTower.Type.ToString();
		Damage.text      = string.Format("{0}-{1}", clickedTower.Tower.MinDamage, clickedTower.Tower.MaxDamage);
		Range.text       = clickedTower.Tower.Range.ToString();
		FireRate.text    = clickedTower.Tower.FireRate.ToString();
		TowerLevel.text  = clickedTower.Tower.Level.ToString();
		UpgradeCost.text = clickedTower.Tower.UpgradeCost.ToString();
		UpgradeType.text = clickedTower.Tower.Upgrade.ToString();

		// Zavřeme všechny specifická GUI, kdyžtak se níže zobrazí
		BeamStatsHolder.SetActive(false);
		HunterStatsHolder.SetActive(false);
		AlchemyStatsHolder.SetActive(false);
		MineStatsHolder.SetActive(false);

		switch (clickedTower.Tower.Type)
		{
			case eTower.TowerType.BeamTower:
				BeamStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxIgnoresLight").text = ((BeamTower)clickedTower.Tower).LightIgnore.ToString();
				BeamStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxIgnoresShadow").text = ((BeamTower)clickedTower.Tower).ShadowIgnore.ToString();
				BeamStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxOnHitResReduction").text = ((BeamTower)clickedTower.Tower).OnHitResReduction.ToString();

				// zobrazíme specifické GUI
				BeamStatsHolder.SetActive(true);
				break;

			case eTower.TowerType.HunterTower:
				HunterStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxIgnoresArmor").text = ((HunterTower)clickedTower.Tower).ArmorIgnore.ToString();
				HunterStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxOnHitArmorReduction").text = ((HunterTower)clickedTower.Tower).OnHitArmorReduction.ToString();
				HunterStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxAoEDamage").text = 
					string.Format("{0}-{1}", ((HunterTower)clickedTower.Tower).MinAoeDamage,
					((HunterTower)clickedTower.Tower).MaxAoeDamage);

				// zobrazíme specifické GUI
				HunterStatsHolder.SetActive(true);
				break;

			case eTower.TowerType.AlchemyTower:
				AlchemyStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxPoisonDamage").text = ((AlchemyTower)clickedTower.Tower).PoisonDamage.ToString();
				AlchemyStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxOnHitHealthReduction").text = ((AlchemyTower)clickedTower.Tower).OnHitHealthReduction.ToString();
				AlchemyStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxOnHitRegenReduction").text = ((AlchemyTower)clickedTower.Tower).OnHitRegenReduction.ToString();
				AlchemyStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxCharmChance").text = ((AlchemyTower)clickedTower.Tower).CharmChance.ToString();

				// zobrazíme specifické GUI
				AlchemyStatsHolder.SetActive(true);
				break;

			case eTower.TowerType.GoldMine:
				MineStatsHolder.GetComponentsInChildren<Text>().FirstOrDefault(
					x => x.gameObject.name == "tbxGoldPerSec").text = ((GoldMine)clickedTower.Tower).GoldPerSec.ToString();

				// zobrazíme specifické GUI
				MineStatsHolder.SetActive(true);
				break;
		}
    }

    public void Destory()
    {
        // free the tile under the tower
        // we must get tower position of a parent (tower holder), parent holds tower position
        // in order to tile size
        tgMap.Map.FindTile(x => x.Position == new Vector2(
            clickedTower.transform.parent.position.x / tgMap.tileSize, -clickedTower.transform.parent.position.y / tgMap.tileSize))
            .Type = eTileType.Texture.Empty;

        Destroy(clickedTower.gameObject);

        // disable gui (tower no longer exists)
        gameObject.SetActive(false);
    }

	public void Upgrade()
	{
		if (gameMaster.Coins >= clickedTower.Tower.UpgradeCost)
		{
			// Než zavoláme level UP, tak musíme zjistit zda neupragudejeme ze 3 lvl na 4
			// čili, zda nemusíme zvolit jaký speciální upgrade tower získá
			// musíme to u tower nastavit, než dojde na metodu LevelUp()
			if (clickedTower.Tower.Level == 3 && !(clickedTower.Tower is GoldMine))
			{
				// Nastavíme texty v GUI a 
				switch (clickedTower.Tower.Type)
				{
					case eTower.TowerType.BeamTower:
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnFirstUpgradeText").text = "Light";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnSecondUpgradeText").text = "Shadow";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnThirdUpgradeText").text = "Suppressing";
						break;

					case eTower.TowerType.HunterTower:
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnFirstUpgradeText").text = "Crossbow";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnSecondUpgradeText").text = "Balista";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnThirdUpgradeText").text = "Catapult";
						break;

					case eTower.TowerType.AlchemyTower:
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnFirstUpgradeText").text = "Poison";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnSecondUpgradeText").text = "Weakening";
						UpgradeGUI.GetComponentsInChildren<Text>().FirstOrDefault(
							x => x.gameObject.name == "btnThirdUpgradeText").text = "Charming";
						break;
				}

				UpgradeGUI.SetActive(true);

                // Samotný upgrade se provede až v reakci na klik jednoho ze 3 buttonů
                // viz. metody níže
            }
			else
			{
                gameMaster.Coins -= clickedTower.Tower.UpgradeCost;
                clickedTower.LevelUp();
            }

			if (TbxWarning.activeSelf)
				TbxWarning.SetActive(false);
		}
		else
			TbxWarning.SetActive(true);

		// re-set values
		SetValues();
	}


	public void FirstUpgrade()
	{
		// nastavíme první upgrade
		switch (clickedTower.Tower.Type)
		{
			case eTower.TowerType.BeamTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.LightBeam;
				break;

			case eTower.TowerType.HunterTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Crossbow;
				break;

			case eTower.TowerType.AlchemyTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Poison;
				break;
		}

        gameMaster.Coins -= clickedTower.Tower.UpgradeCost;
        clickedTower.LevelUp();

        // re-set values
        SetValues();

        UpgradeGUI.SetActive(false);
    }

	public void SecondUpgrade()
	{
		// nastavíme druhý upgrade
		switch (clickedTower.Tower.Type)
		{
			case eTower.TowerType.BeamTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.ShadowBeam;
				break;

			case eTower.TowerType.HunterTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Balista;
				break;

			case eTower.TowerType.AlchemyTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Weakening;
				break;
		}

        gameMaster.Coins -= clickedTower.Tower.UpgradeCost;
        clickedTower.LevelUp();

        // re-set values
        SetValues();

        UpgradeGUI.SetActive(false);
    }

	public void ThirdUpgrade()
	{
		// nastavíme třetí upgrade
		switch (clickedTower.Tower.Type)
		{
			case eTower.TowerType.BeamTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.SupressingBeam;
				break;

			case eTower.TowerType.HunterTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Catapult;
				break;

			case eTower.TowerType.AlchemyTower:
				clickedTower.Tower.Upgrade = eTower.UpgradeType.Charming;
				break;
		}

        gameMaster.Coins -= clickedTower.Tower.UpgradeCost;
        clickedTower.LevelUp();

        // re-set values
        SetValues();

        UpgradeGUI.SetActive(false);
    }

	public void CloseUpgrade()
	{
		UpgradeGUI.SetActive(false);
	}

	public void OnLeftClick(Tile tile)
    {
		if (tile.CanBuildOn && !tile.IsOccupied)
		{
			// clicked on a free spot (hide GUI)
			gameObject.SetActive(false);
		}
		else if (tile.IsOccupied)
		{
			// get all towers
			List<GameObject> towers = GameObject.FindGameObjectsWithTag("Tower").ToList();
			// search for a tower on current tile
			GameObject tower = towers.Find(
				x => x.transform.position.x == tile.Position.x * tgMap.tileSize
					 &&
					 x.transform.position.y == -tile.Position.y * tgMap.tileSize);

			// if tower is null, were clicked on a mine so don't show tower GUI
			if (tower != null)
			{
				clickedTower = tower.GetComponentInChildren<TowerBehaviour>();

				gameObject.SetActive(true);
				//hide warning
				TbxWarning.SetActive(false);
				SetValues();
			}
		}
		else
			gameObject.SetActive(false);
	}

    public void OnHover(Tile tile)
	{
		if (tile.CanBuildOn && !tile.IsOccupied)
		{
			if (towerHighlighter.HighlighterEnabled())
				towerHighlighter.DisableHighlighter();
		}
		else if (tile.IsOccupied)
		{
			// get all towers
			List<GameObject> towers = GameObject.FindGameObjectsWithTag("Tower").ToList();
			// search for a tower on current tile
			GameObject tower = towers.Find(
				x => x.transform.position.x == tile.Position.x * tgMap.tileSize
					 &&
					 x.transform.position.y == -tile.Position.y * tgMap.tileSize);

			// if tower is null, were hovered on a mine so don't set tower hover 
			if (tower != null)
			{
				// get hovered tower script
				hoveredTower = tower.GetComponentInChildren<TowerBehaviour>();

				// another tower was hovered
				// or prev tower is null so hovered tower is first hovered oject
				if (prevHoveredTower != hoveredTower)
				{
					// set range highlighter
					towerHighlighter.SetHighlighter(hoveredTower.Tower.Range, hoveredTower.transform.position);
				}
				else if (!towerHighlighter.HighlighterEnabled())
					towerHighlighter.SetHighlighter(hoveredTower.Tower.Range, hoveredTower.transform.position);

				prevHoveredTower = hoveredTower;
			}
		}
		else if (towerHighlighter.HighlighterEnabled())
			towerHighlighter.DisableHighlighter();
	}
}
