﻿using UnityEngine;
using UnityEngine.SceneManagement;

public class MenuBehaviour : MonoBehaviour
{
    [SerializeField] Canvas gameSettings;
    [SerializeField] Canvas settings;
    [SerializeField] Canvas mainMenu;
    private GameInfo gameInfo;

    void Start()
    {
        gameSettings.enabled = false;
        settings.enabled = false;

        gameInfo = GameObject.Find("_GameInfo").GetComponent<GameInfo>();
    }

    public void ShowGameSettings()
    {
        // show game info and hide main menu
        gameSettings.enabled = true;
        settings.enabled = false;
        mainMenu.enabled = false;
    }

    public void ShowSettings()
    {
        settings.enabled = true;
        gameSettings.enabled = false;
        mainMenu.enabled = false;
    }

    public void ShowMainMenu()
    {
        // show main menu and hide game info
        mainMenu.enabled = true;
        gameSettings.enabled = false;
        settings.enabled = false;
    }

    public void StartGame()
    {
        // Start the game
        if (gameInfo.WavesCount > 0 && gameInfo.Coins > 0)
		    SceneManager.LoadScene("Map");
    }

    public void Quit()
    {
		#if UNITY_EDITOR
		UnityEditor.EditorApplication.isPlaying = false;
		#endif

		Application.Quit();
    }

	public void SetWavesCount(string input)
	{
		gameInfo.WavesCount = int.Parse(input);
	}

	public void SetCoins(string input)
	{
		gameInfo.Coins = int.Parse(input);
	}

    public void WaveSizeToggled(int index)
    {
        switch (index)
        {
            case 1:
                gameInfo.WaveSizeMin = 2;
                gameInfo.WaveSizeMax = 8;
                break;
            case 2:
                gameInfo.WaveSizeMin = 4;
                gameInfo.WaveSizeMax = 16;
                break;
            case 3:
                gameInfo.WaveSizeMin = 8;
                gameInfo.WaveSizeMax = 32;
                break;
        }
    }

    public void EnemiesDifficultyToggled(int index)
    {
        switch (index)
        {
            case 1:
                gameInfo.EnemiesDifficulty = 1;
                break;
            case 2:
                gameInfo.EnemiesDifficulty = 2;
                break;
            case 3:
                gameInfo.EnemiesDifficulty = 3;
                break;
            case 4:
                gameInfo.EnemiesDifficulty = 5;
                break;
        }
    }
}
