using System.Linq;
using UnityEngine;
using System.Collections;
using System;

[RequireComponent(typeof(TileMapGraphics))]
public class TileMapMouse : MonoBehaviour
{
	public GameObject highlightTile;

    private TileMapGraphics tgMap;
    private Vector3 currentTileCoord;
    private Tile tile;
    private TowerGUI towerGUI;
    private BuildGUI buildGUI;

    void Awake()
    {
        // get references before object are disabled by their own scripts
        towerGUI = GameObject.Find("TowerGUI").GetComponent<TowerGUI>();
        buildGUI = GameObject.Find("BuildGUI").GetComponent<BuildGUI>();
    }

    void Start()
	{
        tgMap = GetComponent<TileMapGraphics>();
	}

    // Update is called once per frame
    void Update () 
    {
		Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
		RaycastHit hitInfo;
		
		if (GetComponent<Collider>().Raycast(ray, out hitInfo, Mathf.Infinity)) 
        {
            int x = Mathf.FloorToInt(hitInfo.point.x / tgMap.tileSize);
            int y = Mathf.FloorToInt(hitInfo.point.y / tgMap.tileSize);

            // activate higlight tile, mouse is inside the map
            highlightTile.SetActive(true);

            currentTileCoord.x = x;
            // +1 becase number (-0,3) is -1 in int, but it should be 0 coord of tile, so
            // we have to compensate it by adding +1
            currentTileCoord.y = y + 1;

            highlightTile.transform.position = new Vector3(currentTileCoord.x * tgMap.tileSize,
                currentTileCoord.y * tgMap.tileSize, 0);
        }
		else
		{
            // deactive higlight tile, mouse is outside the map
            highlightTile.SetActive(false);
		}
		
        // click on a tile (right mouse button)
        // show a dialog window over a tile according to it's type
	    if (highlightTile.activeSelf)
	    {
			// Currently hovered tile
			// coord Y muset be negated, bacause it's negative and tile positions in list are positive
			tile = tgMap.Map.GetTile((int)currentTileCoord.x, (int)-currentTileCoord.y);

            towerGUI.OnHover(tile);

            // right mouse click
            // GUI appears only when not prev. GUI from other key is visible
	        if (Input.GetMouseButtonDown(1))
	        {
                buildGUI.Show(tile);
	        }

            // left mouse click
            // GUI appears only when not prev. GUI from other key is visible
	        if (Input.GetMouseButtonDown(0))
	        {
                towerGUI.OnLeftClick(tile);
            }
        }
	}
}
