﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(SpriteRenderer))]
public class TowerRangeHighlighter : MonoBehaviour 
{

    // make a new texture with a properties of hovered tower (it's range)
    // 50 * ((2 * range) + 1) -> width, height
    // 2 * range + 1 so we get all range tiles
    // for example when range is 2
    // we have field of 5x5 range tiles around tower
    // so (2 * 2) + 1 = 5

    public Sprite HighlightSprite;
    private Texture2D texture;
    private SpriteRenderer spriteRenderer;
    private Transform tra;
    private TileMapGraphics tgMap;

	// Use this for initialization
	void Start ()
	{
	    spriteRenderer = GetComponent<SpriteRenderer>();
	    tgMap = GameObject.Find("TileMap").GetComponent<TileMapGraphics>();
	    tra = transform;
	}

    public void SetHighlighter(float range, Vector2 position)
    {
        // create new texture
        texture = new Texture2D((int)(tgMap.tileResolution * ((range * 2) + 1)), (int)(tgMap.tileResolution * ((range * 2) + 1)));

        float leftOver = ((range * 2) + 1) - (int)((range * 2) + 1);
        // first create texture for whole blocks
        for (int y = 0; y < (int)((range * 2) + 1); y++)
        {
            if (leftOver != 0)
            {
                // right left overs (only whole segments)
                texture.SetPixels(HighlightSprite.texture.width * (int)((range * 2) + 1), HighlightSprite.texture.height * y,
                    (int)(HighlightSprite.texture.width * leftOver), HighlightSprite.texture.height,
                    HighlightSprite.texture.GetPixels(0, 0, (int)(HighlightSprite.texture.width * leftOver),
                    HighlightSprite.texture.height));
            }

            for (int x = 0; x < (int)((range * 2) + 1); x++)
            {
                // set each tile in texture
                texture.SetPixels(HighlightSprite.texture.width * x, HighlightSprite.texture.height * y,
                    HighlightSprite.texture.width, HighlightSprite.texture.height,
                    HighlightSprite.texture.GetPixels());

                if (leftOver != 0)
                {
                    // top left overs (only whole segments)
                    texture.SetPixels(HighlightSprite.texture.width * x, HighlightSprite.texture.height * (int)((range * 2) + 1),
                        HighlightSprite.texture.width, (int)(HighlightSprite.texture.height * leftOver),
                        HighlightSprite.texture.GetPixels(0, 0, HighlightSprite.texture.width,
                        (int)(HighlightSprite.texture.height * leftOver)));
                }
            }
        }

        // now add the last segmengt (last piece in top-right corner)
        // it's not a whole piece
        if (leftOver != 0)
        {
            texture.SetPixels(HighlightSprite.texture.width * (int)((range * 2) + 1), HighlightSprite.texture.height * (int)((range * 2) + 1),
                        (int)(HighlightSprite.texture.width * leftOver), (int)(HighlightSprite.texture.height * leftOver),
                        HighlightSprite.texture.GetPixels(0, 0, (int)(HighlightSprite.texture.width * leftOver),
                        (int)(HighlightSprite.texture.height * leftOver)));
        }

        // set the new texture
        texture.Apply();
        spriteRenderer.sprite = Sprite.Create(texture, new Rect(0, 0, texture.width, texture.height),
            new Vector2(0.5f, 0.5f));

        // position highlighter
        tra.position = new Vector3(position.x, position.y);

        // enable renderer again
        spriteRenderer.enabled = true;

        Debug.Log("New hover range texture was set");
    }

    public void DisableHighlighter()
    {
        spriteRenderer.enabled = false;
    }

    public bool HighlighterEnabled()
    {
        return spriteRenderer.enabled;
    }
}
