﻿using System.ComponentModel;

/// <summary>
/// Ancestor to all towers
/// </summary>
public abstract class Tower : INotifyPropertyChanged
{
	public bool BasicUpgradeEnabled
	{
		get { return Level <= 3; }
	}

	private float range;
	public float Range
	{
		get { return range; }
		protected set { range = value; ValueChanged("Range"); }
	}

	public int Level { get; set; }
	public int MinDamage { get; set; }
	public int MaxDamage { get; set; }
	public float FireRate { get; set; }
	public int UpgradeCost { get; set; }
	public eTower.UpgradeType Upgrade { get; set; }
	public eTower.TowerType Type { get; set; }

	public event PropertyChangedEventHandler PropertyChanged;

	public Tower(float range, int minDamage, int maxDamage, float fireRate,
		int upgradeCost, eTower.TowerType type)
	{
		Level       = 0;
		Upgrade     = eTower.UpgradeType.Basic;
		Range       = range;
		MinDamage   = minDamage;
		MaxDamage   = maxDamage;
		FireRate    = fireRate;
		UpgradeCost = upgradeCost;
		Type        = type;
	}

	private void ValueChanged(string property)
	{
		if (PropertyChanged != null)
			PropertyChanged(this, new PropertyChangedEventArgs(property));
	}
}

