﻿public static class AlchemyTowerUpgrades
{
	// Level 0 stats
	public static int MinDamage = 10;
	public static int MaxDamage = 14;
	public static float Range = 1;
	public static float FireRate = 0.5f;
	public static int PoisonDamage = 0;
	public static int OnHitHealthReduction = 2;
	public static int OnHitRegenReduction = 0;
	public static int CharmChance = 0;
	public static int BuildCost = 120;

    public static int LvlOneUpgradeCost = 180;

    // Doladit metodu pro všechny typy (alchemy toweru)
    public static AlchemyTowerUpgrade GetUpgrade(AlchemyTower tower)
	{
		switch (tower.Upgrade)
		{
			case eTower.UpgradeType.Basic:
				switch (tower.Level)
				{
					case 1:
						return new AlchemyTowerUpgrade(10, 18, 1, 0.6f, 0, 4, 0, 0);
					case 2:
						return new AlchemyTowerUpgrade(27, 33, 1.1f, 0.7f, 0, 8, 0, 0);
					case 3:
						return new AlchemyTowerUpgrade(36, 42, 1.1f, 0.75f, 0, 16, 0, 0);
				}
				break;

			case eTower.UpgradeType.Poison:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new AlchemyTowerUpgrade(45, 55, 1.2f, 1f, 25, 0, 0, 0);
				else
					return new AlchemyTowerUpgrade(
						(int)(tower.MinDamage * 1.28f), (int)(tower.MaxDamage * 1.28f),
						tower.Range * 1.08f, tower.FireRate * 1.15f,
						(int)(tower.PoisonDamage * 1.3f), 0, 0, 0);

			case eTower.UpgradeType.Weakening:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new AlchemyTowerUpgrade(42, 52, 1.2f, 1f, 0, 15, 20, 0);
				else
					return new AlchemyTowerUpgrade(
						(int)(tower.MinDamage * 1.32f), (int)(tower.MaxDamage * 1.32f),
						tower.Range * 1.1f, tower.FireRate * 1.15f,
						0, tower.OnHitHealthReduction + 15,
						tower.OnHitRegenReduction + 20, 0);

			case eTower.UpgradeType.Charming:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new AlchemyTowerUpgrade(45, 50, 1.2f, 1f, 0, 0, 0, 40);
				else
					return new AlchemyTowerUpgrade(
						(int)(tower.MinDamage * 1.33f), (int)(tower.MaxDamage * 1.33f),
						tower.Range * 1.08f, tower.FireRate * 1.15f,
						0, 0, 0, (int)(tower.CharmChance * 1.05f));
		}

		return null;
	}

    public static int GetNextLevelUpgradeCost(Tower tower)
    {
        switch (tower.Upgrade)
        {
            case eTower.UpgradeType.Basic:
                switch (tower.Level + 1)
                {
                    case 1:
                        return LvlOneUpgradeCost;
                    case 2:
                        return 290;
                    case 3:
                        return 400;
                    case 4:
                        // First specific upgrade cost, same for all upgrades
                        return 900;
                }
                break;

            case eTower.UpgradeType.Poison:
                    return (int)((tower.UpgradeCost * 1.55f) * 1.55f);

            case eTower.UpgradeType.Weakening:
                    return (int)((tower.UpgradeCost * 1.55f) * 1.55f);

            case eTower.UpgradeType.Charming:
                    return (int)((tower.UpgradeCost * 1.6f) * 1.6f);
        }

        return 0;
    }
}
