﻿
using System;

public static class BeamTowerUpgrades
{
	// Level 0 stats
	public static int MinDamage         = 10;
	public static int MaxDamage         = 16;
	public static float Range           = 1;
	public static float FireRate        = 0.5f;
	public static int LightIgnore       = 10;
	public static int ShadowIgnore      = 10;
	public static int OnHitResReduction = 0;
	public static int BuildCost         = 150;

    public static int LvlOneUpgradeCost = 200;

	// Doladit metodu pro všechny typy (beam toweru)
	public static BeamTowerUpgrade GetUpgrade(BeamTower tower)
	{
		switch (tower.Upgrade)
		{
			case eTower.UpgradeType.Basic:
				switch (tower.Level)
				{
					case 1:
						return new BeamTowerUpgrade(20, 28, 1.1f, 0.55f, 20, 20, 0);
					case 2:
						return new BeamTowerUpgrade(33, 43, 1.2f, 0.6f, 30, 30, 0);
					case 3:
						return new BeamTowerUpgrade(49, 61, 1.3f, 0.65f, 40, 40, 0);
				}
				break;

			case eTower.UpgradeType.LightBeam:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new BeamTowerUpgrade(90, 105, 1.4f, 0.7f, 20, 40, 0);
				else
					return new BeamTowerUpgrade(
						(int)(tower.MinDamage * 1.5f), (int)(tower.MaxDamage * 1.5f), 
						tower.Range * 1.1f, tower.FireRate * 1.05f, 
						(int)(20 - (Math.Sqrt(tower.Level) * tower.Level)),
                        (int)(40 + (Math.Sqrt(tower.Level) * tower.Level)),
                        0);

			case eTower.UpgradeType.ShadowBeam:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new BeamTowerUpgrade(90, 105, 1.4f, 0.7f, 40, 20, 0);
				else
					return new BeamTowerUpgrade(
						(int)(tower.MinDamage * 1.5f), (int)(tower.MaxDamage * 1.5f),
                        tower.Range * 1.1f, tower.FireRate * 1.05f,
                        (int)(40 + (Math.Sqrt(tower.Level) * tower.Level)),
                        (int)(20 - (Math.Sqrt(tower.Level) * tower.Level)),
                        0);

			case eTower.UpgradeType.SupressingBeam:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new BeamTowerUpgrade(90, 105, 1.4f, 0.75f, 30, 30, 4);
				else
					return new BeamTowerUpgrade(
						(int)(tower.MinDamage * 1.5f), (int)(tower.MaxDamage * 1.5f),
						tower.Range * 1.08f, tower.FireRate * 1.08f,
                        (int)(30 + (Math.Sqrt(tower.Level) * tower.Level)),
                        (int)(30 + (Math.Sqrt(tower.Level) * tower.Level)),
                        tower.OnHitResReduction + 6);
		}

		return null;
	}

    public static int GetNextLevelUpgradeCost(Tower tower)
    {
        switch (tower.Upgrade)
        {
            case eTower.UpgradeType.Basic:
                switch (tower.Level + 1)
                {
                    case 1:
                        return LvlOneUpgradeCost;
                    case 2:
                        return 300;
                    case 3:
                        return 450;
                    case 4:
                        // First specific upgrade cost, same for all upgrades
                        return 1000;
                }
                break;

            case eTower.UpgradeType.LightBeam:
                    return (int)(tower.UpgradeCost * 1.6f);

            case eTower.UpgradeType.ShadowBeam:
                    return (int)(tower.UpgradeCost * 1.6f);

            case eTower.UpgradeType.SupressingBeam:
                    return (int)(tower.UpgradeCost * 1.65f);
        }

        return 0;
    }
}
