﻿using System;

public static class HunterTowerUpgrades
{
	// Level 0 stats
	public static int MinDamage = 8;
	public static int MaxDamage = 12;
	public static float Range = 1;
	public static float FireRate = 0.6f;
	public static int ArmorIgnore = 0;
	public static int OnHitArmorReduction = 2;
	public static int MinAoeDamage = 0;
	public static int MaxAoeDamage = 0;
	public static int BuildCost = 100;

    public static int LvlOneUpgradeCost = 140;

    // Doladit metodu pro všechny typy (hunter toweru)
    public static HunterTowerUpgrade GetUpgrade(HunterTower tower)
	{
		switch (tower.Upgrade)
		{
			case eTower.UpgradeType.Basic:
				switch (tower.Level)
				{
					case 1:
						return new HunterTowerUpgrade(14, 19, 1.05f, 0.7f, 0, 4, 0, 0);
					case 2:
						return new HunterTowerUpgrade(20, 24, 1.1f, 0.8f, 0, 8, 0, 0);
					case 3:
						return new HunterTowerUpgrade(26, 31, 1.15f, 0.9f, 0, 16, 0, 0);
				}
				break;

			case eTower.UpgradeType.Crossbow:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new HunterTowerUpgrade(55, 65, 1.2f, 1f, 0, 30, 0, 0);
				else
					return new HunterTowerUpgrade(
						(int)(tower.MinDamage * 1.45f), (int)(tower.MaxDamage * 1.43f),
						tower.Range * 1.08f, tower.FireRate * 1.2f,
						0, tower.OnHitArmorReduction + 20, 0, 0);

			case eTower.UpgradeType.Balista:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new HunterTowerUpgrade(60, 70, 1.2f, 1f, 11, 15, 20, 30);
				else
					return new HunterTowerUpgrade(
						(int)(tower.MinDamage * 1.45f), (int)(tower.MaxDamage * 1.45f),
						tower.Range * 1.08f, tower.FireRate * 1.1f,
						(int)(10 + (Math.Sqrt(tower.Level) * tower.Level)), tower.OnHitArmorReduction + 12,
						(int)(tower.MinAoeDamage * 1.35f), (int)(tower.MaxAoeDamage * 1.35f));

			case eTower.UpgradeType.Catapult:
				// First upgrade level is static, others are generic (increments)
				if (tower.Level == 4)
					return new HunterTowerUpgrade(62, 76, 1.2f, 1f, 20, 20, 40, 60);
				else
					return new HunterTowerUpgrade(
						(int)(tower.MinDamage * 1.45f), (int)(tower.MaxDamage * 1.5f),
						tower.Range * 1.08f, tower.FireRate * 1.1f,
                        (int)(20 + (Math.Sqrt(tower.Level) * tower.Level)), tower.OnHitArmorReduction + 20,
                        (int)(tower.MinAoeDamage * 1.45f), (int)(tower.MaxAoeDamage * 1.5f));
        }

		return null;
	}

    public static int GetNextLevelUpgradeCost(Tower tower)
    {
        switch (tower.Upgrade)
        {
            case eTower.UpgradeType.Basic:
                switch (tower.Level + 1)
                {
                    case 1:
                        return LvlOneUpgradeCost;
                    case 2:
                        return 260;
                    case 3:
                        return 380;
                    case 4:
                        // First specific upgrade cost, same for all upgrades
                        return 810;
                }
                break;

            case eTower.UpgradeType.Crossbow:
                    return (int)((tower.UpgradeCost * 1.5f) * 1.5f);

            case eTower.UpgradeType.Balista:
                    return (int)((tower.UpgradeCost * 1.52f) * 1.52f);

            case eTower.UpgradeType.Catapult:
                    return (int)((tower.UpgradeCost * 1.55f) * 1.55f);
        }

        return 0;
    }
}
