﻿using System.Collections.Generic;
using UnityEngine;
using System.Collections;

[RequireComponent(typeof(EnemyBehaviour))]
public class WaypointAgent : MonoBehaviour
{
    private GameObject waypointsHolder;
    private List<Waypoint> waypoints;
    private Waypoint currentWaypoint;
    private Transform tra;
    private EnemyBehaviour enemyBehaviour;
    private Vector3 prevPos;

	// Use this for initialization
	void Start ()
	{
	    waypointsHolder = GameObject.Find("Waypoints");
        waypoints = GetAllWaypoints();
	    currentWaypoint = waypoints[0];
	    tra = transform;
	    enemyBehaviour = GetComponent<EnemyBehaviour>();
	}
	
	// Update is called once per frame
	void Update () 
    {
	    // moving object to the next waypoint
	    if (!AllWaypointsAchieved())
	    {
            // if object is not on waypoint yet move it
	        if (!ObjectIsAtWaypoint(tra.position, currentWaypoint.Tra.position))
	        {
	            prevPos = tra.position;
	            tra.position = Vector3.MoveTowards(tra.position, currentWaypoint.Tra.position, enemyBehaviour.Enemy.Speed * Time.deltaTime);

                // rotating enemy for better look
	            if (prevPos.y != tra.position.y)
	            {
	                if (tra.rotation.z == 0)
	                    tra.eulerAngles = new Vector3(0, 0, 90);
	            }
	            else
                    tra.eulerAngles = new Vector3(0, 0, 0);
	        }
	        else
	        {
	            currentWaypoint.Achieved = true;
	            if (!AllWaypointsAchieved())
	                currentWaypoint = GetNextWaypoint();
	        }
	    }
	}

    private bool AllWaypointsAchieved()
    {
        return waypoints.FindAll(x => x.Achieved == false).Count == 0;
    }

    private Waypoint GetNextWaypoint()
    {
        return waypoints[waypoints.IndexOf(currentWaypoint) + 1];
    }

    private List<Waypoint> GetAllWaypoints()
    {
        List<Waypoint> newWaypoints = new List<Waypoint>();

        foreach (Transform childTra in waypointsHolder.transform)
            newWaypoints.Add(new Waypoint(childTra));

        return newWaypoints;
    }

    private bool ObjectIsAtWaypoint(Vector2 objectPosition, Vector2 targetPosition)
    {
        return Vector2.Distance(objectPosition, targetPosition) == 0;
    }
}
